@@ -19,6 +19,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "datasource_interface.h"
 #include "datadevicemanager_interface.h"
+#include "clientconnection.h"
 #include "resource_p.h"
 // Qt
 #include <QStringList>
@@ -47,7 +48,6 @@ private:
     void offer(const QString &mimeType);
 
     static void offerCallback(wl_client *client, wl_resource *resource, const char *mimeType);
-    static void destroyCallack(wl_client *client, wl_resource *resource);
 
     const static struct wl_data_source_interface s_interface;
 };
@@ -55,7 +55,7 @@ private:
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 const struct wl_data_source_interface DataSourceInterface::Private::s_interface = {
     offerCallback,
-    destroyCallack
+    resourceDestroyedCallback
 };
 #endif
 
@@ -66,15 +66,6 @@ DataSourceInterface::Private::Private(DataSourceInterface *q, DataDeviceManagerI
 
 DataSourceInterface::Private::~Private() = default;
 
-void DataSourceInterface::Private::destroyCallack(wl_client *client, wl_resource *resource)
-{
-    Q_UNUSED(client)
-    auto p = cast<Private>(resource);
-    wl_resource_destroy(resource);
-    p->resource = nullptr;
-    p->q->deleteLater();
-}
-
 void DataSourceInterface::Private::offerCallback(wl_client *client, wl_resource *resource, const char *mimeType)
 {
     Q_UNUSED(client)
@@ -113,7 +104,11 @@ void DataSourceInterface::requestData(const QString &mimeType, qint32 fd)
 void DataSourceInterface::cancel()
 {
     Q_D();
+    if (!d->resource) {
+        return;
+    }
     wl_data_source_send_cancelled(d->resource);
+    client()->flush();
 }
 
 QStringList DataSourceInterface::mimeTypes() const
